
#include <cstdint>
#include <Pokitto.h>
#include "AWSynthSource.h"
#include "DubSiren.h"

int main() {
    using Pokitto::Core;
    using Pokitto::Display;
    using Pokitto::Buttons;
    
    using AWSynth = Audio::AWSynthSource;
    using AWPatch = Audio::AWPatch;
    
    DubSiren dubsiren;
    auto* dubsiren_cb = AWPatch::makeCallback(&DubSiren::callback);
    auto dubsiren_patch = AWPatch(dubsiren_cb, dubsiren)
        .volume(80).step(8).release(20)
        .amplitudes(AWPatch::Envelope(100,100).loop(1,2));
    
    Core::begin();
    Display::persistence = true;
    Display::setFont(fontTiny);
    
    AWSynth* snd = nullptr;
    
    const char* menu_items[] = {"Volume", "Speed", "Modulation", "Frequency", "Phase", "Release", "Echo", "Reverb"};
    constexpr std::uint32_t ITEM_COUNT = sizeof(menu_items)/sizeof(*menu_items);
    std::uint8_t selected_item = 0;
    
    std::int32_t values[] = {80*(1<<15), 50*(1<<15), 50*(1<<15), 50*(1<<15), 50*(1<<15), 20*(1<<15)};
    bool echo_enabled = false;
    bool reverb_enabled = false;
    
    std::int32_t change_setpoint = 0;
    std::int32_t change = 0;
    
    while(Core::isRunning()) {
        if(!Core::update()) continue;
        
        if(Buttons::pressed(BTN_B)) {
            dubsiren.reset();
            snd = &AWSynth::play<0>(dubsiren_patch);
        }
        else if(Buttons::released(BTN_B)) {
            if(snd) snd->release();
            snd = nullptr;
        }
        
        if(Buttons::pressed(BTN_UP)) {
            selected_item = (selected_item > 0) ? (selected_item - 1) : (ITEM_COUNT - 1);
        }
        if(Buttons::pressed(BTN_DOWN)) {
            selected_item = (selected_item < (ITEM_COUNT - 1)) ? (selected_item + 1) : 0;
        }
        
        if(selected_item < 6) {
            constexpr std::int32_t CHANGE_SPEED = 5.0*(1<<15);
            constexpr std::int32_t CHANGE_ACC = 0.005*(1<<15);
            change_setpoint = (Buttons::rightBtn() ? CHANGE_SPEED : 0) - (Buttons::leftBtn() ? CHANGE_SPEED : 0);
            std::int32_t delta = (change_setpoint != 0) ? ((CHANGE_ACC*(change_setpoint - change) + (1<<14)) >> 15) : 0;
            
            change = (delta != 0) ? (change + delta) : change_setpoint;
            if(change != 0) {
                values[selected_item] += change;
                if(values[selected_item] > 100*(1<<15)) {
                    values[selected_item] = 100*(1<<15);
                }
                else if(values[selected_item] < 0) {
                    values[selected_item] = 0;
                }
                
                switch(selected_item)
                {
                    case 0: dubsiren_patch.volume(values[0]>>15); break;
                    case 1 ... 2: dubsiren.setSpeedAndModulation(values[1]*(0.01/(1<<15)), values[2]*(0.01/(1<<15))); break;
                    case 3: dubsiren.setFrequency(values[3]*(0.01/(1<<15))); break;
                    case 4: dubsiren.setPhase(values[4]*(0.01/(1<<15))); break;
                    case 5: dubsiren_patch.release(values[5]>>15); break;
                }
            }
        }
        else if(selected_item == 6) {
            if(Buttons::pressed(BTN_RIGHT)) echo_enabled = true;
            else if(Buttons::pressed(BTN_LEFT)) echo_enabled = false;
            
            dubsiren.enableEcho(echo_enabled);
        }
        else if(selected_item == 7) {
            if(Buttons::pressed(BTN_RIGHT)) reverb_enabled = true;
            else if(Buttons::pressed(BTN_LEFT)) reverb_enabled = false;
            
            dubsiren.enableReverb(reverb_enabled);
        }
        
        Display::clear();
        Display::print(1, 1, "DUB SIREN");
        for(std::uint32_t idx = 0; idx < ITEM_COUNT; ++idx) {
            if(idx == selected_item) {
                Display::print(1, 10 + 9*idx, ">");
            }
            Display::print(4, 10 + 9*idx, menu_items[idx]);
            
            if(idx < 6) Display::print(59, 10 + 9*idx, std::uint32_t(values[idx]>>15));
            else if(idx == 6) Display::print(59, 10 + 9*idx, echo_enabled ? "on" : "off");
            else if(idx == 7) Display::print(59, 10 + 9*idx, reverb_enabled ? "on" : "off");
        }
        Display::print(72, 1, "B to play");
    }
}
